@@ -7,7 +7,13 @@ from xml.etree.ElementTree import Element, SubElement, tostring  # nosec
 
 import httpx
 
-from .models import ChannelMessageList, ChannelMessageModel, ChannelStatistics
+from .models import (
+    ChannelMessageList,
+    ChannelMessageModel,
+    ChannelModel,
+    ChannelStatistics,
+)
+from .utils import deprecated
 
 if TYPE_CHECKING:
     from .mirth import MirthAPI
@@ -32,19 +38,18 @@ def build_channel_message(raw_data: Optional[str], binary: bool = False) -> str:
 class Channel:
     """Class corresponding to a Mirth channel"""
 
-    def __init__(
-        self,
-        mirth: "MirthAPI",
-        id_: str,
-        name: str,
-        description: Optional[str],
-        revision: str,
-    ) -> None:
+    def __init__(self, mirth: "MirthAPI", id_: str) -> None:
         self.mirth: "MirthAPI" = mirth
         self.id = UUID(id_)
-        self.name = name
-        self.description = description
-        self.revision = revision
+
+    async def get_info(self) -> ChannelModel:
+        """Get basic channel metadata from Mirth.
+
+        Returns:
+            ChannelModel: Channel metadata
+        """
+        response = await self.mirth.get(f"/channels/{self.id}")
+        return ChannelModel.parse_raw(response.text, content_type="xml")
 
     async def get_statistics(self) -> ChannelStatistics:
         """Get basic channel statistics from Mirth.
@@ -124,3 +129,8 @@ class Channel:
             data=message,
             content_type="application/xml",
         )
+
+    # Deprecated function aliases
+    @deprecated
+    async def get(self) -> ChannelModel:  # pylint: disable=missing-function-docstring
+        return await self.get_info()
